@@ -1,8 +1,9 @@
-# Copyright (C) 2001, 2002, 2003 by Intevation GmbH
+# Copyright (C) 2001, 2002, 2003, 2004, 2005, 2007 by Intevation GmbH
 # Authors:
 # Jan-Oliver Wagner <jan@intevation.de>
 # Bernhard Herzog <bh@intevation.de>
 # Jonathan Coles <jonathan@intevation.de>
+# Frank Koormann <frank@intevation.de>
 #
 # This program is free software under the GPL (>=v2)
 # Read the file COPYING coming with GRASS for details.
@@ -11,7 +12,7 @@
 Parser for thuban session files.
 """
 
-__version__ = "$Revision: 1.47 $"
+__version__ = "$Revision: 2826 $"
 
 import string, os
 
@@ -32,7 +33,8 @@ from Thuban.Model.layer import Layer, RasterLayer
 from Thuban.Model.proj import Projection
 from Thuban.Model.range import Range
 from Thuban.Model.classification import Classification, \
-    ClassGroupDefault, ClassGroupSingleton, ClassGroupRange, ClassGroupMap, \
+    ClassGroupDefault, ClassGroupSingleton, ClassGroupRange, \
+    ClassGroupPattern, ClassGroupMap, \
     ClassGroupProperties
 from Thuban.Model.data import DerivedShapeStore, ShapefileStore
 from Thuban.Model.table import DBFTable
@@ -100,12 +102,13 @@ class AttrDesc:
 
 class SessionLoader(XMLReader):
 
-    def __init__(self, db_connection_callback = None):
+    def __init__(self, db_connection_callback = None, 
+                       shapefile_callback = None):
         """Inititialize the Sax handler."""
         XMLReader.__init__(self)
 
         self.db_connection_callback = db_connection_callback
-
+        self.shapefile_callback = shapefile_callback
         self.theSession = None
         self.aMap = None
         self.aLayer = None
@@ -134,6 +137,7 @@ class SessionLoader(XMLReader):
             'clnull'        : ("start_clnull",         "end_clnull"),
             'clpoint'       : ("start_clpoint",        "end_clpoint"),
             'clrange'       : ("start_clrange",        "end_clrange"),
+            'clpattern'     : ("start_clpattern",      "end_clpattern"),
             'cldata'        : ("start_cldata",         "end_cldata"),
             'table'         : ("start_table",          "end_table"),
             'labellayer'    : ("start_labellayer",     None),
@@ -145,7 +149,9 @@ class SessionLoader(XMLReader):
                       "http://thuban.intevation.org/dtds/thuban-0.9.dtd",
                       "http://thuban.intevation.org/dtds/thuban-1.0-dev.dtd",
                       "http://thuban.intevation.org/dtds/thuban-1.0rc1.dtd",
-                      "http://thuban.intevation.org/dtds/thuban-1.0.0.dtd"):
+                      "http://thuban.intevation.org/dtds/thuban-1.0.0.dtd",
+                      "http://thuban.intevation.org/dtds/thuban-1.1-dev.dtd",
+                      "http://thuban.intevation.org/dtds/thuban-1.2.1.dtd"):
             for key, value in dispatchers.items():
                 dispatchers[(xmlns, key)] = value
 
@@ -245,7 +251,7 @@ class SessionLoader(XMLReader):
                                     % (element, d.name))
             elif d.conversion == "filename":
                 value = os.path.abspath(os.path.join(self.GetDirectory(),
-                                                     value))
+                                                     self.encode(value)))
             elif d.conversion == "ascii":
                 value = value.encode("ascii")
             elif d.conversion:
@@ -257,6 +263,51 @@ class SessionLoader(XMLReader):
             normalized[d.name] = value
         return normalized
 
+    def open_shapefile(self, filename):
+        """Open shapefile, with alternative path handling.
+         
+           If a shapefile cannot be opened and an IOError is raised, check for
+           an alternative. This alternative can be specified interactively by
+           the user or taken from a list of (potential) locations, depending on
+           the callback implementation.
+            
+           The alternative is rechecked. If taken from a list the user
+           has to confirm the alternative.
+        """
+
+        # Flag if the alternative path was specified interactively / from list.
+        from_list = 0
+        while 1:
+            try: 
+                store = self.theSession.OpenShapefile(filename)
+                if from_list:
+                    # A valid path has been guessed from a list
+                    # Let the user confirm - or select an alternative.
+                    filename, from_list = self.shapefile_callback(
+                                            filename, "check")
+                    if filename is None:
+                        # Selection cancelled
+                        raise LoadCancelled
+                    elif store.FileName() == filename:
+                        # Proposed file has been accepted
+                        break
+                    else:
+                        # the filename has been changed, try the new file
+                        pass
+                else:
+                    break
+            except IOError:
+                if self.shapefile_callback is not None:
+                    filename, from_list = self.shapefile_callback(
+                                            filename, 
+                                            mode = "search", 
+                                            second_try = from_list)
+                    if filename is None:
+                        raise LoadCancelled
+                else:
+                    raise
+        return store
+
     def start_dbconnection(self, name, qname, attrs):
         attrs = self.check_attrs(name, attrs,
                                  [AttrDesc("id", True),
@@ -300,11 +351,26 @@ class SessionLoader(XMLReader):
                                   AttrDesc("dbconn", True,
                                            conversion = "idref"),
                                   AttrDesc("tablename", True,
+                                           conversion = "ascii"),
+                                  # id_column and geometry_column were
+                                  # newly introduced with thuban-1.1.dtd
+                                  # where they're required.  Since we
+                                  # support the older formats too we
+                                  # have them optional here.
+                                  AttrDesc("id_column", False, "gid",
+                                           conversion = "ascii"),
+                                  AttrDesc("geometry_column", False,
                                            conversion = "ascii")])
-        ID = attrs["id"]
-        db = attrs["dbconn"]
-        tablename = attrs["tablename"]
-        self.idmap[ID] = self.theSession.OpenDBShapeStore(db, tablename)
+        # The default value of geometry_column to use when instantiating
+        # the db shapestore is None which we currently can't easily use
+        # in check_attrs
+        geometry_column = attrs["geometry_column"]
+        if not geometry_column:
+            geometry_column = None
+        dbopen = self.theSession.OpenDBShapeStore
+        self.idmap[attrs["id"]] = dbopen(attrs["dbconn"], attrs["tablename"],
+                                         id_column = attrs["id_column"],
+                                         geometry_column=geometry_column)
 
     def start_fileshapesource(self, name, qname, attrs):
         attrs = self.check_attrs(name, attrs,
@@ -317,7 +383,7 @@ class SessionLoader(XMLReader):
         filetype = attrs["filetype"]
         if filetype != "shapefile":
             raise LoadError("shapesource filetype %r not supported" % filetype)
-        self.idmap[ID] = self.theSession.OpenShapefile(filename)
+        self.idmap[ID] = self.open_shapefile(filename)
 
     def start_derivedshapesource(self, name, qname, attrs):
         attrs = self.check_attrs(name, attrs,
@@ -428,7 +494,8 @@ class SessionLoader(XMLReader):
         if attrs.has_key((None, "shapestore")):
             store = self.idmap[attrs[(None, "shapestore")]]
         else:
-            store = self.theSession.OpenShapefile(filename)
+            store = self.open_shapefile(filename)
+
         self.aLayer = layer_class(title, store,
                                   fill = fill, stroke = stroke,
                                   lineWidth = stroke_width,
@@ -444,20 +511,34 @@ class SessionLoader(XMLReader):
         filename = os.path.join(self.GetDirectory(), filename)
         filename = self.encode(filename)
         visible  = self.encode(attrs.get((None, 'visible'), "true")) != "false"
+        opacity  = float(attrs.get((None, 'opacity'), "1"))
+        masktype = str(attrs.get((None, 'masktype'), "alpha"))
 
-        self.aLayer = layer_class(title, filename, visible = visible)
+        masktypes = {"none": layer_class.MASK_NONE,
+                     "bit":  layer_class.MASK_BIT,
+                     "alpha": layer_class.MASK_ALPHA}
+
+        self.aLayer = layer_class(title, filename, 
+                                  visible = visible,
+                                  opacity = opacity,
+                                  masktype = masktypes[masktype])
 
     def end_rasterlayer(self, name, qname):
         self.aMap.AddLayer(self.aLayer)
         self.aLayer = None
 
     def start_classification(self, name, qname, attrs):
+    	# field and field_type are optional because the classification
+	# can also be empty, ie. have only a default.
         attrs = self.check_attrs(name, attrs,
-                                 [AttrDesc("field", True),
-                                  AttrDesc("field_type", True)])
+                                 [AttrDesc("field", False),
+                                  AttrDesc("field_type", False)])
+
         field = attrs["field"]
         fieldType = attrs["field_type"]
 
+	if field == "": return # no need to set classification column.
+
         dbFieldType = self.aLayer.GetFieldType(field)
 
         if fieldType != dbFieldType:
@@ -506,11 +587,15 @@ class SessionLoader(XMLReader):
         del self.cl_group, self.cl_prop
 
     def start_clrange(self, name, qname, attrs):
+        attrs = self.check_attrs(name, attrs,
+                                 [AttrDesc("range", False, None),
+                                  AttrDesc("min", False, None),
+                                  AttrDesc("max", False, None)])
 
-        range = attrs.get((None, 'range'), None)
+        range = attrs['range']
         # for backward compatibility (min/max are not saved)
-        min   = attrs.get((None, 'min'), None)
-        max   = attrs.get((None, 'max'), None)
+        min   = attrs['min']
+        max   = attrs['max']
 
         try:
             if range is not None:
@@ -533,11 +618,26 @@ class SessionLoader(XMLReader):
         self.aLayer.GetClassification().AppendGroup(self.cl_group)
         del self.cl_group, self.cl_prop
 
+
+    def start_clpattern(self, name, qname, attrs):
+        pattern = attrs.get((None, 'pattern'), "")
+
+        self.cl_group = ClassGroupPattern(self.encode(pattern))
+        self.cl_group.SetLabel(self.encode(attrs.get((None, 'label'), "")))
+        self.cl_prop = ClassGroupProperties()
+
+    def end_clpattern(self, name, qname):
+        self.cl_group.SetProperties(self.cl_prop)
+        self.aLayer.GetClassification().AppendGroup(self.cl_group)
+        del self.cl_group, self.cl_prop
+
+
     def start_cldata(self, name, qname, attrs):
         self.cl_prop.SetLineColor(
             parse_color(attrs.get((None, 'stroke'), "None")))
         self.cl_prop.SetLineWidth(
             int(attrs.get((None, 'stroke_width'), "0")))
+        self.cl_prop.SetSize(int(attrs.get((None, 'size'), "5")))
         self.cl_prop.SetFill(parse_color(attrs.get((None, 'fill'), "None")))
 
     def end_cldata(self, name, qname):
@@ -570,7 +670,8 @@ class SessionLoader(XMLReader):
         pass
 
 
-def load_session(filename, db_connection_callback = None):
+def load_session(filename, db_connection_callback = None, 
+                           shapefile_callback = None):
     """Load a Thuban session from the file object file
 
     The db_connection_callback, if given should be a callable object
@@ -583,7 +684,7 @@ def load_session(filename, db_connection_callback = None):
     corrected and perhaps additional parameters like a password or None
     to indicate that the user cancelled.
     """
-    handler = SessionLoader(db_connection_callback)
+    handler = SessionLoader(db_connection_callback, shapefile_callback)
     handler.read(filename)
 
     session = handler.theSession
