@@ -1,4 +1,4 @@
-# Copyright (c) 2003 by Intevation GmbH
+# Copyright (c) 2003, 2004 by Intevation GmbH
 # Authors:
 # Jonathan Coles <jonathan@intevation.de>
 # Frank Koormann <frank.koormann@intevation.de>
@@ -9,12 +9,12 @@
 
 """Projection dialog"""
 
-__version__ = "$Revision: 1.38 $"
-# $Source: /thubanrepository/thuban/Thuban/UI/projdialog.py,v $
-# $Id: projdialog.py,v 1.38 2003/12/05 13:36:10 frank Exp $
+__version__ = "$Revision: 2816 $"
+# $Source$
+# $Id: projdialog.py 2816 2008-01-26 22:21:53Z bernhard $
 
-import os
-from wxPython.wx import *
+import sys, os
+import wx
 
 from Thuban import _
 
@@ -24,6 +24,7 @@ from Thuban.Model.resource import get_user_proj_file, get_system_proj_file, \
                                   read_proj_file, write_proj_file, \
                                   DEFAULT_PROJ_FILE, EPSG_PROJ_FILE, \
                                   EPSG_DEPRECATED_PROJ_FILE
+from Thuban.UI import internal_from_wxstring
 from Thuban.UI.dialogs import NonModalNonParentDialog
 
 from common import ThubanBeginBusyCursor, ThubanEndBusyCursor
@@ -80,89 +81,89 @@ class ProjFrame(NonModalNonParentDialog):
 
     def build_dialog(self):
         """Build the dialog's widgets and set the event handlers"""
-        self.topBox = top_box = wxBoxSizer(wxVERTICAL)
+        self.topBox = top_box = wx.BoxSizer(wx.VERTICAL)
 
-        main_box = wxBoxSizer(wxHORIZONTAL)
-        top_box.Add(main_box, 1, wxALL|wxEXPAND)
+        main_box = wx.BoxSizer(wx.HORIZONTAL)
+        top_box.Add(main_box, 1, wx.ALL|wx.EXPAND)
 
         #
         #    The projection list and associated controls
         #
-        vbox = wxBoxSizer(wxVERTICAL)
-        main_box.Add(vbox, 4, wxALL|wxEXPAND)
+        vbox = wx.BoxSizer(wx.VERTICAL)
+        main_box.Add(vbox, 4, wx.ALL|wx.EXPAND)
 
         #label = wxStaticText(self, -1, _("Available Projections:"))
         #vbox.Add(label, 0, wxLEFT|wxRIGHT|wxTOP, 4)
 
-        hbox = wxBoxSizer(wxHORIZONTAL)
-        vbox.Add(hbox, 1, wxALL|wxEXPAND)
+        hbox = wx.BoxSizer(wx.HORIZONTAL)
+        vbox.Add(hbox, 1, wx.ALL|wx.EXPAND)
         proj_files = [self.load_user_proj(),
                       self.load_system_proj(DEFAULT_PROJ_FILE)]
         self.projection_list = ProjectionList(self, proj_files,
                                               self.receiver.GetProjection())
-        hbox.Add(self.projection_list, 1, wxALL|wxEXPAND|wxADJUST_MINSIZE, 4)
+        hbox.Add(self.projection_list, 1, wx.ALL|wx.EXPAND|wx.ADJUST_MINSIZE, 4)
         self.projection_list.Subscribe(PROJ_SELECTION_CHANGED,
                                        self.proj_selection_changed)
 
         # Projection List specific actions (Import/Export/Remove)
-        buttons = wxBoxSizer(wxVERTICAL)
-        hbox.Add(buttons, 0, wxALL)
-        self.button_import = wxButton(self, ID_PROJ_IMPORT, _("Import..."))
-        EVT_BUTTON(self, ID_PROJ_IMPORT, self._OnImport)
-        buttons.Add(self.button_import, 1, wxALL|wxEXPAND, 4)
-        self.button_export = wxButton(self, ID_PROJ_EXPORT, _("Export..."))
-        EVT_BUTTON(self, ID_PROJ_EXPORT, self._OnExport)
-        buttons.Add(self.button_export, 1, wxALL|wxEXPAND, 4)
-        buttons.Add(20, 20, 0, wxEXPAND, 0)
-        self.button_remove = wxButton(self, ID_PROJ_REMOVE, _("Remove"))
-        EVT_BUTTON(self, ID_PROJ_REMOVE, self._OnRemove)
-        buttons.Add(self.button_remove, 1, wxALL|wxEXPAND, 4)
+        buttons = wx.BoxSizer(wx.VERTICAL)
+        hbox.Add(buttons, 0, wx.ALL)
+        self.button_import = wx.Button(self, ID_PROJ_IMPORT, _("Import..."))
+        self.Bind(wx.EVT_BUTTON, self._OnImport, id=ID_PROJ_IMPORT)
+        buttons.Add(self.button_import, 1, wx.ALL|wx.EXPAND, 4)
+        self.button_export = wx.Button(self, ID_PROJ_EXPORT, _("Export..."))
+        self.Bind(wx.EVT_BUTTON, self._OnExport, id=ID_PROJ_EXPORT)
+        buttons.Add(self.button_export, 1, wx.ALL|wx.EXPAND, 4)
+        buttons.Add( (20, 20), 0, wx.EXPAND, 0)
+        self.button_remove = wx.Button(self, ID_PROJ_REMOVE, _("Remove"))
+        self.Bind(wx.EVT_BUTTON, self._OnRemove, id=ID_PROJ_REMOVE)
+        buttons.Add(self.button_remove, 1, wx.ALL|wx.EXPAND, 4)
 
-        buttons.Add(20, 20, 0, wxEXPAND, 0)
-        label = wxStaticText(self, -1, _("Show EPSG:"))
-        buttons.Add(label, 0, wxLEFT|wxRIGHT|wxTOP, 4)
-        self.check_epsg = wxCheckBox(self, -1, _("Normal"))
-        EVT_CHECKBOX(self, self.check_epsg.GetId(), self._OnShowEPSG)
-        buttons.Add(self.check_epsg, 1, wxALL|wxEXPAND, 4)
-        self.check_epsg_depr = wxCheckBox(self, -1, _("Deprecated"))
-        EVT_CHECKBOX(self, self.check_epsg_depr.GetId(), self._OnShowEPSG)
-        buttons.Add(self.check_epsg_depr, 1, wxALL|wxEXPAND, 4)
+        buttons.Add( (20, 20), 0, wx.EXPAND, 0)
+        label = wx.StaticText(self, -1, _("Show EPSG:"))
+        buttons.Add(label, 0, wx.LEFT|wx.RIGHT|wx.TOP, 4)
+        self.check_epsg = wx.CheckBox(self, -1, _("Normal"))
+        self.Bind(wx.EVT_CHECKBOX, self._OnShowEPSG, id=self.check_epsg.GetId())
+        buttons.Add(self.check_epsg, 1, wx.ALL|wx.EXPAND, 4)
+        self.check_epsg_depr = wx.CheckBox(self, -1, _("Deprecated"))
+        self.Bind(wx.EVT_CHECKBOX, self._OnShowEPSG, id=self.check_epsg_depr.GetId())
+        buttons.Add(self.check_epsg_depr, 1, wx.ALL|wx.EXPAND, 4)
 
         # The file path
-        self.projfilepath = wxStaticText(self, -1, "")
-        vbox.Add(self.projfilepath, 0, wxALL|wxEXPAND)
+        self.projfilepath = wx.StaticText(self, -1, "")
+        vbox.Add(self.projfilepath, 0, wx.ALL|wx.EXPAND)
 
         #
         #   The projection editor part
         #
-        self.edit_box = wxStaticBox(self, -1, _("Edit"))
-        sizer_edit = wxStaticBoxSizer(self.edit_box, wxHORIZONTAL)
-        main_box.Add(sizer_edit, 5, wxALL|wxEXPAND)
+        self.edit_box = wx.StaticBox(self, -1, _("Edit"))
+        sizer_edit = wx.StaticBoxSizer(self.edit_box, wx.HORIZONTAL)
+        main_box.Add(sizer_edit, 5, wx.ALL|wx.EXPAND)
 
         # Projection Specific Entries (Name/Projection)
-        self.sizer_projctrls = wxBoxSizer(wxVERTICAL)
-        sizer_edit.Add(self.sizer_projctrls, 1, wxALL|wxEXPAND)
+        self.sizer_projctrls = wx.BoxSizer(wx.VERTICAL)
+        sizer_edit.Add(self.sizer_projctrls, 1, wx.ALL|wx.EXPAND)
 
-        hbox = wxBoxSizer(wxHORIZONTAL)
-        self.sizer_projctrls.Add(hbox, 0, wxALL|wxEXPAND)
-        label = wxStaticText(self, -1, _("Name:"))
-        hbox.Add(label, 0, wxALL|wxALIGN_CENTER_VERTICAL, 4)
-        self.projname = wxTextCtrl(self, ID_PROJ_PROJNAME, "")
-        EVT_TEXT(self, ID_PROJ_PROJNAME, self._OnProjName)
-        hbox.Add(self.projname, 1, wxALL|wxEXPAND, 4)
+        hbox = wx.BoxSizer(wx.HORIZONTAL)
+        self.sizer_projctrls.Add(hbox, 0, wx.ALL|wx.EXPAND)
+        label = wx.StaticText(self, -1, _("Name:"))
+        hbox.Add(label, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 4)
+        self.projname = wx.TextCtrl(self, ID_PROJ_PROJNAME, "")
+        self.Bind(wx.EVT_TEXT, self._OnProjName, id=ID_PROJ_PROJNAME)
+        hbox.Add(self.projname, 1, wx.ALL|wx.EXPAND, 4)
 
-        hbox = wxBoxSizer(wxHORIZONTAL)
-        self.sizer_projctrls.Add(hbox, 0, wxALL|wxEXPAND)
-        label = wxStaticText(self, -1, _("Projection:"))
-        hbox.Add(label, 0, wxALL|wxALIGN_CENTER_VERTICAL, 4)
-        self.projchoice = wxChoice(self, ID_PROJ_PROJCHOICE)
+        hbox = wx.BoxSizer(wx.HORIZONTAL)
+        self.sizer_projctrls.Add(hbox, 0, wx.ALL|wx.EXPAND)
+        label = wx.StaticText(self, -1, _("Projection:"))
+        hbox.Add(label, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 4)
+        self.projchoice = wx.Choice(self, ID_PROJ_PROJCHOICE)
         self.projchoice.SetSelection(0)
-        EVT_CHOICE(self, ID_PROJ_PROJCHOICE, self._OnProjChoice)
-        hbox.Add(self.projchoice, 1, wxALL|wxEXPAND, 4)
+        self.Bind(wx.EVT_CHOICE, self._OnProjChoice, id=ID_PROJ_PROJCHOICE)
+        hbox.Add(self.projchoice, 1, wx.ALL|wx.EXPAND, 4)
         # Fill the projection choice list.
         self.nbsizer = NotebookLikeSizer()
         self.sizer_projctrls.Add(self.nbsizer, 1,
-                                 wxALL|wxEXPAND|wxADJUST_MINSIZE, 3)
+                                 wx.ALL|wx.EXPAND|wx.ADJUST_MINSIZE, 3)
         self.projection_panels = []
         self.projchoice.Append(_("<Unknown>"), "")
         for proj_type, name, cls in self.projection_panel_defs:
@@ -178,37 +179,37 @@ class ProjFrame(NonModalNonParentDialog):
         self.nbsizer.Add(self.unknown_projection_panel)
 
         # Projection Specific actions (New/Save/Add)
-        buttons = wxBoxSizer(wxVERTICAL)
-        sizer_edit.Add(buttons, 0, wxALL)
-        self.button_new = wxButton(self, ID_PROJ_NEW, _("New"))
-        EVT_BUTTON(self, ID_PROJ_NEW, self._OnNew)
-        buttons.Add(self.button_new, 0, wxEXPAND|wxALL, 4)
-        self.button_add = wxButton(self, ID_PROJ_ADDTOLIST, _("Add to List"))
-        EVT_BUTTON(self, ID_PROJ_ADDTOLIST, self._OnAddToList)
-        buttons.Add(self.button_add, 0, wxEXPAND|wxALL, 4)
-        buttons.Add(20, 20, 0, wxEXPAND, 0)
-        self.button_save = wxButton(self, ID_PROJ_SAVE,_("Update"))
-        EVT_BUTTON(self, ID_PROJ_SAVE, self._OnSave)
-        buttons.Add(self.button_save, 0, wxEXPAND|wxALL|wxALIGN_BOTTOM, 4)
+        buttons = wx.BoxSizer(wx.VERTICAL)
+        sizer_edit.Add(buttons, 0, wx.ALL)
+        self.button_new = wx.Button(self, ID_PROJ_NEW, _("New"))
+        self.Bind(wx.EVT_BUTTON, self._OnNew, id=ID_PROJ_NEW)
+        buttons.Add(self.button_new, 0, wx.EXPAND|wx.ALL, 4)
+        self.button_add = wx.Button(self, ID_PROJ_ADDTOLIST, _("Add to List"))
+        self.Bind(wx.EVT_BUTTON, self._OnAddToList, id=ID_PROJ_ADDTOLIST)
+        buttons.Add(self.button_add, 0, wx.EXPAND|wx.ALL, 4)
+        buttons.Add( (20, 20), 0, wx.EXPAND, 0)
+        self.button_save = wx.Button(self, ID_PROJ_SAVE,_("Update"))
+        self.Bind(wx.EVT_BUTTON, self._OnSave, id=ID_PROJ_SAVE)
+        buttons.Add(self.button_save, 0, wx.EXPAND|wx.ALL|wx.ALIGN_BOTTOM, 4)
 
         #
         # Main Action buttons (Try/Revert/OK/Close)
         #
-        buttons = wxBoxSizer(wxHORIZONTAL)
-        top_box.Add(buttons, 0, wxALL|wxALIGN_RIGHT, 10)
-        self.button_try = wxButton(self, wxID_APPLY, _("Try"))
-        EVT_BUTTON(self, wxID_APPLY, self.OnApply)
-        buttons.Add(self.button_try, 0, wxRIGHT, 10)
-        self.button_revert = wxButton(self, ID_PROJ_REVERT, _("Revert"))
-        EVT_BUTTON(self, ID_PROJ_REVERT, self._OnRevert)
-        buttons.Add(self.button_revert, 0, wxRIGHT, 10)
-        self.button_ok = wxButton(self, wxID_OK, _("OK"))
-        EVT_BUTTON(self, wxID_OK, self.OnOK)
+        buttons = wx.BoxSizer(wx.HORIZONTAL)
+        top_box.Add(buttons, 0, wx.ALL|wx.ALIGN_RIGHT, 10)
+        self.button_try = wx.Button(self, wx.ID_APPLY, _("Try"))
+        self.Bind(wx.EVT_BUTTON, self.OnApply, id=wx.ID_APPLY)
+        buttons.Add(self.button_try, 0, wx.RIGHT, 10)
+        self.button_revert = wx.Button(self, ID_PROJ_REVERT, _("Revert"))
+        self.Bind(wx.EVT_BUTTON, self._OnRevert, id=ID_PROJ_REVERT)
+        buttons.Add(self.button_revert, 0, wx.RIGHT, 10)
+        self.button_ok = wx.Button(self, wx.ID_OK, _("OK"))
+        self.Bind(wx.EVT_BUTTON, self.OnOK, id=wx.ID_OK)
         self.button_ok.SetDefault()
-        buttons.Add(self.button_ok, 0, wxRIGHT, 10)
-        self.button_close = wxButton(self, wxID_CANCEL, _("Close"))
-        EVT_BUTTON(self, wxID_CANCEL, self.OnCancel)
-        buttons.Add(self.button_close, 0, wxRIGHT, 10)
+        buttons.Add(self.button_ok, 0, wx.RIGHT, 10)
+        self.button_close = wx.Button(self, wx.ID_CANCEL, _("Close"))
+        self.Bind(wx.EVT_BUTTON, self.OnCancel, id=wx.ID_CANCEL)
+        buttons.Add(self.button_close, 0, wx.RIGHT, 10)
 
 
         #
@@ -303,22 +304,24 @@ class ProjFrame(NonModalNonParentDialog):
         add them to the user ProjFile object and write the user file
         back to disk.
         """
-        dlg = wxFileDialog(self, _("Import"), style = wxOPEN)
+        dlg = wx.FileDialog(self, _("Import"),
+                self.parent.application.Path("projection"), style = wx.OPEN)
 
-        if dlg.ShowModal() == wxID_OK:
-            path = dlg.GetPath()
+        if dlg.ShowModal() == wx.ID_OK:
+            path = internal_from_wxstring(dlg.GetPath())
 
             ThubanBeginBusyCursor()
             try:
                 try:
                     projFile, warnings = read_proj_file(path)
                 except IOError, (errno, errstr):
-                    self.__ShowError(dlg.GetPath(), errstr)
+                    self.__ShowError(path, errstr)
                 else:
                     self.show_warnings(_("Warnings"), path, warnings)
                     for proj in projFile.GetProjections():
                         self.__usrProjFile.Add(proj)
                     self.write_proj_file(self.__usrProjFile)
+                    self.parent.application.SetPath("projection", path)
             finally:
                 ThubanEndBusyCursor()
         dlg.Destroy()
@@ -332,14 +335,18 @@ class ProjFrame(NonModalNonParentDialog):
         sel = self.projection_list.selected_projections()
         assert len(sel) != 0, "button should be disabled"
 
-        dlg = wxFileDialog(self, _("Export"), style=wxSAVE|wxOVERWRITE_PROMPT)
+        dlg = wx.FileDialog(self, _("Export"),
+                self.parent.application.Path("projection"),
+                style=wx.SAVE|wx.OVERWRITE_PROMPT)
 
-        if dlg.ShowModal() == wxID_OK:
-            proj_file = ProjFile(dlg.GetPath())
+        if dlg.ShowModal() == wx.ID_OK:
+            path = internal_from_wxstring(dlg.GetPath())
+            proj_file = ProjFile(path)
             for proj, pf in sel:
                 if proj is not None:
                     proj_file.Add(proj)
             self.write_proj_file(proj_file)
+            self.parent.application.SetPath("projection", path)
 
         dlg.Destroy()
 
@@ -380,10 +387,10 @@ class ProjFrame(NonModalNonParentDialog):
         self.__VerifyButtons()
 
     def __ShowError(self, filename, errstr):
-        wxMessageDialog(self,
+        wx.MessageDialog(self,
             _("The following error occured:\n") +
             filename + "\n" + errstr,
-            _("Error"), wxOK | wxICON_ERROR).ShowModal()
+            _("Error"), wx.OK | wx.ICON_ERROR).ShowModal()
 
     def __VerifyButtons(self):
         """Update button sensitivity"""
@@ -525,10 +532,14 @@ class ProjFrame(NonModalNonParentDialog):
     def _show_proj_panel(self, panel_class):
         """Show the panel as the projection panel"""
         if panel_class is UnknownProjPanel:
+            self.button_ok.Disable()
+            self.button_try.Disable()
             self.edit_box.Disable()
             self.nbsizer.Activate(self.unknown_projection_panel)
             self.curProjPanel = self.unknown_projection_panel
         else:
+            self.button_ok.Enable(True)
+            self.button_try.Enable(True)
             self.edit_box.Enable(True)
             self.unknown_projection_panel.Hide()
             for panel in self.projection_panels:
@@ -564,8 +575,10 @@ class ProjFrame(NonModalNonParentDialog):
         # self.curProjPanel should always contain the most relevant data
         # for a projection
         if self.curProjPanel is not None:
-            return Projection(self.curProjPanel.GetParameters(),
-                              self.projname.GetValue())
+            parameters = \
+                map(internal_from_wxstring,self.curProjPanel.GetParameters())
+            if parameters is not None:
+                return Projection(parameters, self.projname.GetValue())
 
         return None
 
@@ -576,7 +589,7 @@ class ProjFrame(NonModalNonParentDialog):
         with get_user_proj_file and cache it in self.__usrProjFile.
 
         Show a busy cursor while loading the file.
-    
+
         If the file is not available, leave a note to the console.
         """
         if self.__usrProjFile is None:
@@ -585,6 +598,7 @@ class ProjFrame(NonModalNonParentDialog):
                 projfile, warnings = get_user_proj_file()
                 if warnings:
                     sys.stderr.write("".join(warnings))
+                    sys.stderr.write("\n")
                 self.__usrProjFile = projfile
             finally:
                 ThubanEndBusyCursor()
@@ -627,13 +641,13 @@ class ProjFrame(NonModalNonParentDialog):
 
 
 
-class ProjPanel(wxPanel):
+class ProjPanel(wx.Panel):
     """Base class for all projection panels."""
 
     def __init__(self, parent):
-        wxPanel.__init__(self, parent, -1)
+        wx.Panel.__init__(self, parent, -1)
 
-        self.__ellps = wxChoice(self, -1)
+        self.__ellps = wx.Choice(self, -1)
         self.ellpsData = [("", _("<Unknown>")),
                           ("airy"  , _("Airy")),
                           ("bessel", _("Bessel 1841")),
@@ -647,20 +661,20 @@ class ProjPanel(wxPanel):
             self.__ellps.Append(name, tag)
 
         self.__ellps.SetSelection(0)
-        
+
     def _DoLayout(self, childPanel = None):
 
-        panelSizer = wxBoxSizer(wxVERTICAL)
+        panelSizer = wx.BoxSizer(wx.VERTICAL)
 
-        sizer = wxBoxSizer(wxHORIZONTAL)
-        sizer.Add(wxStaticText(self, -1, _("Ellipsoid:")), 0,
-                                    wxALL|wxALIGN_CENTER_VERTICAL, 4)
-        sizer.Add(self.__ellps, 1, wxALL|wxALIGN_CENTER_VERTICAL, 4)
-        panelSizer.Add(sizer, 0, wxALL|wxEXPAND, 4)
+        sizer = wx.BoxSizer(wx.HORIZONTAL)
+        sizer.Add(wx.StaticText(self, -1, _("Ellipsoid:")), 0,
+                                    wx.ALL|wx.ALIGN_CENTER_VERTICAL, 4)
+        sizer.Add(self.__ellps, 1, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 4)
+        panelSizer.Add(sizer, 0, wx.ALL|wx.EXPAND, 4)
 
         if childPanel is not None:
-            panelSizer.Add(childPanel, 0, wxEXPAND, 0)
-            
+            panelSizer.Add(childPanel, 0, wx.EXPAND, 0)
+
         self.SetAutoLayout(1)
         self.SetSizer(panelSizer)
         panelSizer.Fit(self)
@@ -707,14 +721,14 @@ class UnknownProjPanel(ProjPanel):
                         "display a configuration panel.\n\n"
                         "The unidentified set of parameters is:\n\n")
 
-        self.__textbox = wxTextCtrl(self, -1, self.__text, size=(100,200),
-                            style=wxTE_READONLY|wxTE_MULTILINE|wxTE_LINEWRAP)
+        self.__textbox = wx.TextCtrl(self, -1, self.__text, size=(100,200),
+                            style=wx.TE_READONLY|wx.TE_MULTILINE|wx.TE_LINEWRAP)
         self._DoLayout()
 
     def _DoLayout(self):
-        sizer = wxBoxSizer(wxVERTICAL)
+        sizer = wx.BoxSizer(wx.VERTICAL)
 
-        sizer.Add(self.__textbox, 0, wxALL|wxEXPAND, 4)
+        sizer.Add(self.__textbox, 0, wx.ALL|wx.EXPAND, 4)
 
         ProjPanel._DoLayout(self, sizer)
 
@@ -737,27 +751,27 @@ class TMPanel(ProjPanel):
     def __init__(self, parent, receiver):
         ProjPanel.__init__(self, parent)
 
-        self.__latitude = wxTextCtrl(self, ID_TMPANEL_LAT)
-        self.__longitude = wxTextCtrl(self, ID_TMPANEL_LONG)
-        self.__falseEast = wxTextCtrl(self, ID_TMPANEL_FASLE_EAST)
-        self.__falseNorth = wxTextCtrl(self, ID_TMPANEL_FALSE_NORTH)
-        self.__scale = wxTextCtrl(self, ID_TMPANEL_SCALE)
+        self.__latitude = wx.TextCtrl(self, ID_TMPANEL_LAT)
+        self.__longitude = wx.TextCtrl(self, ID_TMPANEL_LONG)
+        self.__falseEast = wx.TextCtrl(self, ID_TMPANEL_FASLE_EAST)
+        self.__falseNorth = wx.TextCtrl(self, ID_TMPANEL_FALSE_NORTH)
+        self.__scale = wx.TextCtrl(self, ID_TMPANEL_SCALE)
 
         self._DoLayout()
 
     def _DoLayout(self):
 
-        sizer = wxFlexGridSizer(4, 2, 0, 0)
-        sizer.Add(wxStaticText(self, -1, _("Latitude:")), 0, wxALL, 4)
-        sizer.Add(self.__latitude, 0, wxALL, 4)
-        sizer.Add(wxStaticText(self, -1, _("Longitude:")), 0, wxALL, 4)
-        sizer.Add(self.__longitude, 0, wxALL, 4)
-        sizer.Add(wxStaticText(self, -1, _("False Easting:")), 0, wxALL, 4)
-        sizer.Add(self.__falseEast, 0, wxALL, 4)
-        sizer.Add(wxStaticText(self, -1, _("False Northing:")), 0, wxALL, 4)
-        sizer.Add(self.__falseNorth, 0, wxALL, 4)
-        sizer.Add(wxStaticText(self, -1, _("Scale Factor:")), 0, wxALL, 4)
-        sizer.Add(self.__scale, 0, wxALL, 4)
+        sizer = wx.FlexGridSizer(4, 2, 0, 0)
+        sizer.Add(wx.StaticText(self, -1, _("Latitude:")), 0, wx.ALL, 4)
+        sizer.Add(self.__latitude, 0, wx.ALL, 4)
+        sizer.Add(wx.StaticText(self, -1, _("Longitude:")), 0, wx.ALL, 4)
+        sizer.Add(self.__longitude, 0, wx.ALL, 4)
+        sizer.Add(wx.StaticText(self, -1, _("False Easting:")), 0, wx.ALL, 4)
+        sizer.Add(self.__falseEast, 0, wx.ALL, 4)
+        sizer.Add(wx.StaticText(self, -1, _("False Northing:")), 0, wx.ALL, 4)
+        sizer.Add(self.__falseNorth, 0, wx.ALL, 4)
+        sizer.Add(wx.StaticText(self, -1, _("Scale Factor:")), 0, wx.ALL, 4)
+        sizer.Add(self.__scale, 0, wx.ALL, 4)
 
         ProjPanel._DoLayout(self, sizer)
 
@@ -806,24 +820,24 @@ class UTMPanel(ProjPanel):
 
         self.receiver = receiver
 
-        self.__zone = wxSpinCtrl(self, ID_UTMPANEL_ZONE, "1", min=1, max=60)
-        self.__propButton = wxButton(self, ID_UTMPANEL_PROP, _("Propose"))
-        self.__south = wxCheckBox(self, ID_UTMPANEL_SOUTH, 
+        self.__zone = wx.SpinCtrl(self, ID_UTMPANEL_ZONE, "1", min=1, max=60)
+        self.__propButton = wx.Button(self, ID_UTMPANEL_PROP, _("Propose"))
+        self.__south = wx.CheckBox(self, ID_UTMPANEL_SOUTH,
                                   _("Southern Hemisphere"))
 
         self._DoLayout()
 
-        EVT_BUTTON(self, ID_UTMPANEL_PROP, self._OnPropose)
+        self.Bind(wx.EVT_BUTTON, self._OnPropose, id=ID_UTMPANEL_PROP)
 
     def _DoLayout(self):
 
-        sizer = wxBoxSizer(wxVERTICAL)
-        psizer = wxBoxSizer(wxHORIZONTAL)
-        psizer.Add(wxStaticText(self, -1, _("Zone:")), 0, wxALL, 4)
-        psizer.Add(self.__zone, 0, wxALL, 4)
-        psizer.Add(self.__propButton, 0, wxALL, 4)
-        sizer.Add(psizer, 0, wxALL, 4)
-        sizer.Add(self.__south, 0, wxALL, 4)
+        sizer = wx.BoxSizer(wx.VERTICAL)
+        psizer = wx.BoxSizer(wx.HORIZONTAL)
+        psizer.Add(wx.StaticText(self, -1, _("Zone:")), 0, wx.ALL, 4)
+        psizer.Add(self.__zone, 0, wx.ALL, 4)
+        psizer.Add(self.__propButton, 0, wx.ALL, 4)
+        sizer.Add(psizer, 0, wx.ALL, 4)
+        sizer.Add(self.__south, 0, wx.ALL, 4)
 
         ProjPanel._DoLayout(self, sizer)
 
@@ -855,57 +869,57 @@ class UTMPanel(ProjPanel):
         """
         bb = self.receiver.BoundingBox()
         if bb is None:
-            dlg = wxMessageDialog(self,
+            dlg = wx.MessageDialog(self,
                     _("Can not propose: No bounding box found."),
                     _("Projection: Propose UTM Zone"),
-                    wxOK | wxICON_INFORMATION)
+                    wx.OK | wx.ICON_INFORMATION)
             dlg.CenterOnParent()
             result = dlg.ShowModal()
             dlg.Destroy()
             return
 
         dlg = UTMProposeZoneDialog(self, self.receiver.BoundingBox())
-        if dlg.ShowModal() == wxID_OK:
+        if dlg.ShowModal() == wx.ID_OK:
             self.__zone.SetValue(dlg.GetProposedZone())
 
-class LCCPanel(ProjPanel): 
+class LCCPanel(ProjPanel):
     """Projection Panel for Lambert Conic Conformal."""
 
     def __init__(self, parent, receiver):
         ProjPanel.__init__(self, parent)
-        
-        self.__fspLatitude = wxTextCtrl(self, -1)
-        self.__sspLatitude = wxTextCtrl(self, -1)
-        self.__meridian    = wxTextCtrl(self, -1)
-        self.__originLat   = wxTextCtrl(self, -1)
-        self.__falseEast   = wxTextCtrl(self, -1)
-        self.__falseNorth  = wxTextCtrl(self, -1)
+
+        self.__fspLatitude = wx.TextCtrl(self, -1)
+        self.__sspLatitude = wx.TextCtrl(self, -1)
+        self.__meridian    = wx.TextCtrl(self, -1)
+        self.__originLat   = wx.TextCtrl(self, -1)
+        self.__falseEast   = wx.TextCtrl(self, -1)
+        self.__falseNorth  = wx.TextCtrl(self, -1)
 
         self._DoLayout()
 
     def _DoLayout(self):
 
-        sizer = wxFlexGridSizer(6, 2, 0, 0)
-        sizer.Add(wxStaticText(self, -1, 
+        sizer = wx.FlexGridSizer(6, 2, 0, 0)
+        sizer.Add(wx.StaticText(self, -1,
             _("Latitude of first standard parallel:")))
-        sizer.Add(self.__fspLatitude, 0, wxALL, 4)
-        sizer.Add(wxStaticText(self, -1,
+        sizer.Add(self.__fspLatitude, 0, wx.ALL, 4)
+        sizer.Add(wx.StaticText(self, -1,
             _("Latitude of second standard parallel:")))
-        sizer.Add(self.__sspLatitude, 0, wxALL, 4)
-        sizer.Add(wxStaticText(self, -1, _("Central Meridian:")))
-        sizer.Add(self.__meridian, 0, wxALL, 4)
-        sizer.Add(wxStaticText(self, -1, _("Latitude of origin:")))
-        sizer.Add(self.__originLat, 0, wxALL, 4)
-        sizer.Add(wxStaticText(self, -1, _("False Easting:")))
-        sizer.Add(self.__falseEast, 0, wxALL, 4)
-        sizer.Add(wxStaticText(self, -1, _("False Northing:")))
-        sizer.Add(self.__falseNorth, 0, wxALL, 4)
+        sizer.Add(self.__sspLatitude, 0, wx.ALL, 4)
+        sizer.Add(wx.StaticText(self, -1, _("Central Meridian:")))
+        sizer.Add(self.__meridian, 0, wx.ALL, 4)
+        sizer.Add(wx.StaticText(self, -1, _("Latitude of origin:")))
+        sizer.Add(self.__originLat, 0, wx.ALL, 4)
+        sizer.Add(wx.StaticText(self, -1, _("False Easting:")))
+        sizer.Add(self.__falseEast, 0, wx.ALL, 4)
+        sizer.Add(wx.StaticText(self, -1, _("False Northing:")))
+        sizer.Add(self.__falseNorth, 0, wx.ALL, 4)
 
         ProjPanel._DoLayout(self, sizer)
 
     def GetProjName(self):
         return _("Lambert Conic Conformal")
-        
+
     def SetProjection(self, proj):
         self.__fspLatitude.SetValue(proj.GetParameter("lat_1"))
         self.__sspLatitude.SetValue(proj.GetParameter("lat_2"))
@@ -917,7 +931,7 @@ class LCCPanel(ProjPanel):
         ProjPanel.SetProjection(self, proj)
 
     def GetParameters(self):
-        params = ["proj=lcc", 
+        params = ["proj=lcc",
                   "lat_1=" + self.__fspLatitude.GetValue(),
                   "lat_2=" + self.__sspLatitude.GetValue(),
                   "lat_0=" + self.__originLat.GetValue(),
@@ -938,7 +952,7 @@ class LCCPanel(ProjPanel):
 
         ProjPanel.Clear(self)
 
-class GeoPanel(ProjPanel): 
+class GeoPanel(ProjPanel):
     """Projection Panel for a Geographic Projection."""
 
     def __init__(self, parent, receiver):
@@ -947,7 +961,7 @@ class GeoPanel(ProjPanel):
         self.__choices = [(_("Degrees"), "0.017453"),
                           (_("Radians"), "1")]
 
-        self.__scale = wxChoice(self, -1)
+        self.__scale = wx.Choice(self, -1)
         for choice, value in self.__choices:
             self.__scale.Append(choice, value)
 
@@ -955,7 +969,7 @@ class GeoPanel(ProjPanel):
 
     def GetProjName(self):
         return _("Geographic")
-        
+
     def SetProjection(self, proj):
         value = proj.GetParameter("to_meter")
         for i in range(len(self.__choices)):
@@ -965,7 +979,7 @@ class GeoPanel(ProjPanel):
         ProjPanel.SetProjection(self, proj)
 
     def GetParameters(self):
-        params = ["proj=latlong", 
+        params = ["proj=latlong",
                   "to_meter=%s" % self.__scale.GetClientData(
                                   self.__scale.GetSelection())]
 
@@ -976,11 +990,11 @@ class GeoPanel(ProjPanel):
         ProjPanel.Clear(self)
 
     def _DoLayout(self):
-        sizer = wxBoxSizer(wxHORIZONTAL)
+        sizer = wx.BoxSizer(wx.HORIZONTAL)
 
-        sizer.Add(wxStaticText(self, -1, _("Source Data is in: ")), 
-                  0, wxALL|wxALIGN_CENTER_VERTICAL, 4)
-        sizer.Add(self.__scale, 1, wxEXPAND|wxALL, 4)
+        sizer.Add(wx.StaticText(self, -1, _("Source Data is in: ")),
+                  0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 4)
+        sizer.Add(self.__scale, 1, wx.EXPAND|wx.ALL, 4)
 
         self.__scale.SetSelection(0)
 
@@ -989,13 +1003,13 @@ class GeoPanel(ProjPanel):
 
 ID_UTM_PROPOSE_ZONE_DIALOG_TAKE   = 4001
 ID_UTM_PROPOSE_ZONE_DIALOG_CANCEL = 4002
-class UTMProposeZoneDialog(wxDialog):
+class UTMProposeZoneDialog(wx.Dialog):
 
     """Propose a sensible Zone considering the current map extent."""
 
     def __init__(self, parent, (x, y, x2, y2)):
-        wxDialog.__init__(self, parent, -1, _("Projection: Propose UTM Zone"),
-                          wxDefaultPosition, wxSize(200, 100))
+        wx.Dialog.__init__(self, parent, -1, _("Projection: Propose UTM Zone"),
+                          wx.DefaultPosition, wx.Size(200, 100))
         self.parent = parent
         x = x + 180
         x2 = x2 + 180
@@ -1004,25 +1018,25 @@ class UTMProposeZoneDialog(wxDialog):
         self.dialogLayout()
 
     def dialogLayout(self):
-        topBox = wxBoxSizer(wxVERTICAL)
+        topBox = wx.BoxSizer(wx.VERTICAL)
 
-        textBox = wxBoxSizer(wxVERTICAL)
-        textBox.Add(wxStaticText(self, -1, _("The current map extent center "
+        textBox = wx.BoxSizer(wx.VERTICAL)
+        textBox.Add(wx.StaticText(self, -1, _("The current map extent center "
                                              "lies in UTM Zone")),
-                    0, wxALIGN_CENTER|wxALL, 4)
-        textBox.Add(wxStaticText(self, -1, str(self.proposedZone)),
-                    0, wxALIGN_CENTER|wxALL, 4)
+                    0, wx.ALIGN_CENTER|wx.ALL, 4)
+        textBox.Add(wx.StaticText(self, -1, str(self.proposedZone)),
+                    0, wx.ALIGN_CENTER|wx.ALL, 4)
 
-        topBox.Add(textBox, 1, wxEXPAND|wxALL, 4)
+        topBox.Add(textBox, 1, wx.EXPAND|wx.ALL, 4)
 
-        buttonBox = wxBoxSizer(wxHORIZONTAL)
-        buttonBox.Add(wxButton(self, ID_UTM_PROPOSE_ZONE_DIALOG_TAKE,
-                      _("Take")), 0, wxALL, 4)
-        buttonBox.Add(wxButton(self, ID_UTM_PROPOSE_ZONE_DIALOG_CANCEL,
-                               _("Cancel")), 0, wxALL, 4)
-        topBox.Add(buttonBox, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 10)
-        EVT_BUTTON(self, ID_UTM_PROPOSE_ZONE_DIALOG_TAKE, self.OnTake)
-        EVT_BUTTON(self, ID_UTM_PROPOSE_ZONE_DIALOG_CANCEL, self.OnCancel)
+        buttonBox = wx.BoxSizer(wx.HORIZONTAL)
+        buttonBox.Add(wx.Button(self, ID_UTM_PROPOSE_ZONE_DIALOG_TAKE,
+                      _("Take")), 0, wx.ALL, 4)
+        buttonBox.Add(wx.Button(self, ID_UTM_PROPOSE_ZONE_DIALOG_CANCEL,
+                               _("Cancel")), 0, wx.ALL, 4)
+        topBox.Add(buttonBox, 0, wx.ALIGN_CENTER_HORIZONTAL|wx.ALL, 10)
+        self.Bind(wx.EVT_BUTTON, self.OnTake, id=ID_UTM_PROPOSE_ZONE_DIALOG_TAKE)
+        self.Bind(wx.EVT_BUTTON, self.OnCancel, id=ID_UTM_PROPOSE_ZONE_DIALOG_CANCEL)
 
         self.SetAutoLayout(True)
         self.SetSizer(topBox)
@@ -1030,10 +1044,10 @@ class UTMProposeZoneDialog(wxDialog):
         topBox.SetSizeHints(self)
 
     def OnTake(self, event):
-        self.EndModal(wxID_OK)
+        self.EndModal(wx.ID_OK)
 
     def OnCancel(self, event):
-        self.EndModal(wxID_CANCEL)
+        self.EndModal(wx.ID_CANCEL)
 
     def GetProposedZone(self):
         return self.proposedZone
