@@ -1,4 +1,4 @@
-# Copyright (c) 2002, 2003 by Intevation GmbH
+# Copyright (c) 2002, 2003, 2004, 2005, 2007 by Intevation GmbH
 # Authors:
 # Bernhard Herzog <bh@intevation.de>
 #
@@ -9,9 +9,9 @@
 Test the Layer class
 """
 
-__version__ = "$Revision: 1.29 $"
-# $Source: /thubanrepository/thuban/test/test_layer.py,v $
-# $Id: test_layer.py,v 1.29 2003/11/28 11:37:35 bh Exp $
+__version__ = "$Revision: 2774 $"
+# $Source$
+# $Id: test_layer.py 2774 2007-06-22 22:14:15Z bernhard $
 
 import os
 import unittest
@@ -32,7 +32,8 @@ from Thuban.Model.table import FIELDTYPE_DOUBLE, FIELDTYPE_STRING, MemoryTable
 from Thuban.Model.proj import Projection
 from Thuban.Model.data import DerivedShapeStore
 from Thuban.Model.classification import Classification, ClassGroupSingleton, \
-     ClassGroupRange
+     ClassGroupRange, ClassGroupPattern
+from Thuban.Model.color import Color
 
 import Thuban.Model.resource
 
@@ -190,6 +191,12 @@ class TestLayer(unittest.TestCase, support.FileTestMixin,
         self.assertFloatSeqEqual(layer.ShapesBoundingBox([0]),
                                  (90.0, -8.90043373, 120, 11.1616263))
 
+        # Test a very large bounding box in the query.  Naive inverse
+        # projection will create infs instead of proper coordinate
+        # values and a different result (an empty list instead of [0])
+        shapes = layer.ShapesInRegion((-180, -170, 200, +120))
+        self.assertEquals([s.ShapeID() for s in shapes],[0])
+
     def test_empty_layer(self):
         """Test Layer with empty shape file"""
         # create an empty shape file
@@ -225,6 +232,8 @@ class TestLayer(unittest.TestCase, support.FileTestMixin,
         layer = RasterLayer("Test RasterLayer", filename)
         self.failIf(layer.HasClassification())
         self.failIf(layer.HasShapes())
+        #testing the default for MaskType
+        self.assertEquals(layer.MaskType(), layer.MASK_ALPHA)
         self.assertEquals(layer.GetImageFilename(), os.path.abspath(filename))
         self.assertFloatSeqEqual(layer.BoundingBox(),
                                  [-24.5500000, 63.2833330,
@@ -233,6 +242,18 @@ class TestLayer(unittest.TestCase, support.FileTestMixin,
                                  [-24.5500000, 63.2833330,
                                   -13.4916670, 66.5666670])
 
+        info = layer.ImageInfo()
+        self.failIf(info is None)
+        self.failUnless(info.has_key("nBands"))
+        self.failUnless(info.has_key("Size"))
+        self.failUnless(info.has_key("Driver"))
+        self.failUnless(info.has_key("BandData"))
+
+        self.assertEquals(info["nBands"], 1)
+        self.assertEquals(info["Size"], (5002, 394))
+        self.assertEquals(info["Driver"], "GTiff")
+        self.assertEquals(info["BandData"], [(0.0, 140.0)])
+
     def test_derived_store(self):
         """Test layer with derived store"""
         layer = self.layer = Layer("Test Layer",
@@ -355,9 +376,9 @@ class TestLayerModification(unittest.TestCase, support.SubscriberMixin):
         """
         self.clear_messages()
         self.session = Session("Test session for %s" % self.__class__)
-        filename = os.path.join("..", "Data", "iceland", "political.shp")
+        self.filename = os.path.join("..", "Data", "iceland", "political.shp")
         self.layer = Layer("Test Layer",
-                           self.session.OpenShapefile(filename))
+                           self.session.OpenShapefile(self.filename))
         self.layer.Subscribe(LAYER_LEGEND_CHANGED, self.subscribe_with_params,
                              LAYER_LEGEND_CHANGED)
         self.layer.Subscribe(LAYER_VISIBILITY_CHANGED,
@@ -375,6 +396,9 @@ class TestLayerModification(unittest.TestCase, support.SubscriberMixin):
         self.session = None
         self.clear_messages()
 
+    def build_path(self, filename):
+        return os.path.join("..", "Data", "iceland", filename)
+
     def test_sanity(self):
         """TestLayerModification Sanity Checks"""
         # test default settings
@@ -393,8 +417,8 @@ class TestLayerModification(unittest.TestCase, support.SubscriberMixin):
         # the layer.
         self.failIf(self.layer.WasModified())
 
-    def test_set_classification(self):
-        """Test Layer.SetClassification"""
+    def test_set_classification_numerical(self):
+        """Test Layer.SetClassification numerical"""
         classification = Classification()
         classification.AppendGroup(ClassGroupRange((0.0, 0.1)))
 
@@ -415,21 +439,99 @@ class TestLayerModification(unittest.TestCase, support.SubscriberMixin):
         self.check_messages([(self.layer, LAYER_CHANGED)])
         self.failUnless(self.layer.WasModified())
 
+    def test_set_classification_textual(self):
+        """Test Layer.SetClassification textual"""
+        classification = Classification()
+        classification.AppendGroup(ClassGroupPattern("I"))
 
-#
-# the tree info now contains Color objects which are difficult to test
-#
-#   def test_tree_info(self):
-#       """Test Layer.TreeInfo"""
-#       self.assertEquals(self.layer.TreeInfo(),
-#                         ("Layer 'Test Layer'",
-#                          ['Shown',
-#                           'Shapes: 156',
-#                           ('Extent (lat-lon):'
-#                            ' (-24.5465, 63.2868, -13.4958, 66.5638)'),
-#                           'Shapetype: Polygon',
-#                           'Fill: None',
-#                           'Outline: (0.000, 0.000, 0.000)']))
+        self.layer.SetClassification(classification)
+        self.layer.SetClassificationColumn("POPYCOUN")
+
+        self.check_messages([(self.layer, LAYER_CHANGED),
+                             (self.layer, LAYER_CHANGED)])
+        self.failUnless(self.layer.WasModified())
+
+        self.clear_messages()
+        self.layer.UnsetModified()
+
+        # change only the classification column. This should issue a
+        # LAYER_CHANGED message as well.
+        self.layer.SetClassificationColumn("POPYREG")
+
+        self.check_messages([(self.layer, LAYER_CHANGED)])
+        self.failUnless(self.layer.WasModified())
+
+
+    def test_tree_info(self):
+        """Test Layer.TreeInfo"""
+        self.assertEquals(self.layer.TreeInfo(),
+                          ("Layer 'Test Layer'",
+                           ['Filename: %s' % os.path.abspath(self.filename),
+                            'Shown',
+                            'Shapes: 156',
+                   'Extent (lat-lon): (-24.5465, 63.2868, -13.4958, 66.5638)',
+                            'Shapetype: Polygon',
+                            self.layer.GetClassification()]))
+
+    def test_raster_layer(self):
+        if not Thuban.Model.resource.has_gdal_support():
+            raise support.SkipTest("No gdal support")
+
+
+        filename = self.build_path("island.tif")
+        layer = RasterLayer("Test RasterLayer", filename)
+
+        layer.Subscribe(LAYER_CHANGED, self.subscribe_with_params,
+                        LAYER_CHANGED)
+
+	# default mask type
+        self.assertEquals(layer.MaskType(), layer.MASK_ALPHA)
+
+        layer.SetMaskType(layer.MASK_NONE)
+        self.failIf(layer.MaskType() != layer.MASK_NONE)
+        self.check_messages([(layer, LAYER_CHANGED)])
+        self.clear_messages()
+
+        layer.SetMaskType(layer.MASK_NONE)
+        self.failIf(layer.MaskType() != layer.MASK_NONE)
+        self.check_messages([])
+        self.clear_messages()
+
+        layer.SetMaskType(layer.MASK_BIT)
+        self.failIf(layer.MaskType() != layer.MASK_BIT)
+        self.check_messages([(layer, LAYER_CHANGED)])
+        self.clear_messages()
+
+        layer.SetMaskType(layer.MASK_BIT)
+        self.failIf(layer.MaskType() != layer.MASK_BIT)
+        self.check_messages([])
+        self.clear_messages()
+
+        layer.SetMaskType(layer.MASK_ALPHA)
+        self.failIf(layer.MaskType() != layer.MASK_ALPHA)
+
+        layer.SetOpacity(0)
+        self.assertEquals(layer.Opacity(), 0)
+        layer.SetOpacity(0.5)
+        self.assertEquals(layer.Opacity(), 0.5)
+
+        self.clear_messages()
+        layer.SetOpacity(1)
+        self.assertEquals(layer.Opacity(), 1)
+        self.check_messages([(layer, LAYER_CHANGED)])
+        self.clear_messages()
+
+        self.assertRaises(ValueError, layer.SetOpacity, -0.1)
+        self.assertRaises(ValueError, layer.SetOpacity, 1.1) 
+
+        layer.SetMaskType(layer.MASK_NONE)
+        self.clear_messages()
+        self.assertEquals(layer.Opacity(), 1)
+        self.check_messages([])
+        self.clear_messages()
+
+        self.assertRaises(ValueError, layer.SetMaskType, -1) 
+        self.assertRaises(ValueError, layer.SetMaskType, 4) 
 
 
 if __name__ == "__main__":
