@@ -17,9 +17,20 @@ dir = os.path.join(thubandir, "Lib")
 if os.path.isdir(dir):
     sys.path.insert(0, dir)
 
+# win32 gdal support
+if sys.platform == 'win32':
+    # PYTHONPATH update
+    dir = os.path.join(thubandir, "gdal", "pymod")
+    if os.path.isdir(dir):
+        sys.path.append(dir)
+
+    try:
+        import gdal
+    except ImportError:
+        print "Please update your PATH environment variable to include %s\\gdal\\bin" % thubandir
+
 import Thuban.UI.main
-    
+
 # Start Thuban
-Thuban.UI.main.main()
-
-
+if __name__ == "__main__":
+    Thuban.UI.main.main()
